#include "WaveFileWriter.hpp"
#include "Logger.hpp"

WaveFileWriter::WaveFileWriter(WAVEFORMATEX format, DWORD length, const Logger* logger)
: format_(format), length_(length), file_(NULL), logger_(logger)
{
}

WaveFileWriter::~WaveFileWriter()
{
	close();
}

bool WaveFileWriter::open(const TCHAR* filepath)
{
	if (filepath == NULL)
	{
		// t@CpXw肳ĂȂΕWo͂ΏۂƂ
		file_ = stdout;

		logger_->verbos(_T("o͐Wo͂ɐݒ肵܂B\n"));
	}
	else
	{
		file_ = _tfopen(filepath, _T("wb"));
		if (file_ == NULL)
		{
			logger_->error(_T("o̓t@CJ܂BpXmFĂB\n"));
			return false;
		}

		logger_->verbos(_T("o̓t@CJ܂B\n"));
	}

	// RIFFʎqo
	if (fwrite("RIFF", 1, 4, file_) != 4)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// t@CTCYo
	DWORD fileLength = 4 + 4 + 20 + 4 + length_;
	if (fwrite(&fileLength, 4, 1, file_) != 1)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// WAVEʎqfmt`Ñ^Oo
	if (fwrite("WAVEfmt ", 1, 8, file_) != 8)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// fmt`ÑTCYo
	DWORD size = 16;
	if (fwrite(&size, 4, 1, file_) != 1)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// tH[}bg^Oo
	if (fwrite(&format_.wFormatTag, 2, 1, file_) != 1)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// `lo
	if (fwrite(&format_.nChannels, 2, 1, file_) != 1)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// TvO[go
	if (fwrite(&format_.nSamplesPerSec, 4, 1, file_) != 1)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// f[^[go
	if (fwrite(&format_.nAvgBytesPerSec, 4, 1, file_) != 1)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// ubNTCYo
	if (fwrite(&format_.nBlockAlign, 2, 1, file_) != 1)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// rbgo
	if (fwrite(&format_.wBitsPerSample, 2, 1, file_) != 1)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// data`Ñ^Oo
	if (fwrite("data", 1, 4, file_) != 4)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	// g`f[^̃TCYo
	if (fwrite(&length_, 4, 1, file_) != 1)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	return true;
}

void WaveFileWriter::close()
{
	if (file_ != NULL)
	{
		logger_->verbos(_T("f[^oI܂B\n"));
		if (file_ != stdout)
		{
			fclose(file_);
			logger_->verbos(_T("o̓t@C܂B\n"));
		}

		file_ = NULL;
	}
}

bool WaveFileWriter::write(const void* buffer, DWORD size)
{
	// data`Ñ^Oo
	if (fwrite(buffer, 1, size, file_) != size)
	{
		logger_->error(_T("f[^oɃG[܂BĎsĂB\n"));
		return false;
	}

	return true;
}
